@@ -10,6 +10,7 @@ extern "C" {
 #include "sentry_envelope.h"
 #include "sentry_logger.h"
 #include "sentry_logs.h"
+#include "sentry_metrics.h"
 #include "sentry_options.h"
 #ifdef SENTRY_PLATFORM_WINDOWS
 #    include "sentry_os.h"
@@ -356,10 +357,13 @@ sentry__crashpad_handler(int signum, siginfo_t *info, ucontext_t *user_context)
                 crash_event, nullptr, options->before_send_data);
         }
 
-        // Flush logs in a crash-safe manner before crash handling
+        // Flush logs and metrics in a crash-safe manner before crash handling
         if (options->enable_logs) {
             sentry__logs_flush_crash_safe();
         }
+        if (options->enable_metrics) {
+            sentry__metrics_flush_crash_safe();
+        }
 
         should_dump = !sentry_value_is_null(crash_event);
 
@@ -539,6 +543,9 @@ crashpad_backend_startup(
 
     std::vector<std::string> arguments { "--no-rate-limit" };
 
+    char report_id[37];
+    sentry_uuid_as_string(&data->crash_event_id, report_id);
+
     // Initialize database first, flushing the consent later on as part of
     // `sentry_init` will persist the upload flag.
     data->db = crashpad::CrashReportDatabase::Initialize(database).release();
@@ -564,7 +571,8 @@ crashpad_backend_startup(
         minidump_url ? minidump_url : "", proxy_url, annotations, arguments,
         /* restartable */ true,
         /* asynchronous_start */ false, attachments, screenshot,
-        options->crashpad_wait_for_upload, crash_reporter, crash_envelope);
+        options->crashpad_wait_for_upload, crash_reporter, crash_envelope,
+        report_id);
     sentry_free(minidump_url);
 
 #ifdef SENTRY_PLATFORM_WINDOWS
