@@ -757,6 +757,26 @@ sentry_options_get_logs_with_attributes(const sentry_options_t *opts)
     return opts->logs_with_attributes;
 }
 
+void
+sentry_options_set_enable_metrics(sentry_options_t *opts, int enable_metrics)
+{
+    opts->enable_metrics = !!enable_metrics;
+}
+
+int
+sentry_options_get_enable_metrics(const sentry_options_t *opts)
+{
+    return opts->enable_metrics;
+}
+
+void
+sentry_options_set_before_send_metric(sentry_options_t *opts,
+    sentry_before_send_metric_function_t func, void *user_data)
+{
+    opts->before_send_metric_func = func;
+    opts->before_send_metric_data = user_data;
+}
+
 #ifdef SENTRY_PLATFORM_LINUX
 
 sentry_handler_strategy_t
