@@ -68,6 +68,9 @@ struct sentry_options_s {
     // takes the first varg as a `sentry_value_t` object containing attributes
     // if no custom attributes are to be passed, use `sentry_value_new_object()`
     bool logs_with_attributes;
+    bool enable_metrics;
+    sentry_before_send_metric_function_t before_send_metric_func;
+    void *before_send_metric_data;
 
     /* everything from here on down are options which are stored here but
        not exposed through the options API */
